from os import path
import os
import pkg_resources
import swimlane_platform
from swimlane_platform.lib.functions import find_remote_sibling
from questions_groups import web_ssl_questions, ssl_questions
from swimlane_platform.lib import ValidationException, \
    Configuration, \
    info_function_start_finish, \
    debug_function_args, \
    BaseWithLog, \
    names, \
    DockerComposeManager, \
    PathExistsValidator, \
    VersionValidator, \
    DockerManager, \
    DockerComposeFileManager
from swimlane_platform.shared_steps import enable_mongo_ssl
from swimlane_platform.lib.version_manager import pypi_version_to_semver
from swimlane_platform.install_steps import enable_web_ssl
from swimlane_platform.add_file_encryption_key import run as run_add_file_encryption
from swimlane_platform.install_steps.setup_swimlane_environment import run as run_setup_environment
from swimlane_platform.install_steps.set_permissions import run as run_set_permissions
from swimlane_platform.shared_steps import enable_turbine
from swimlane_platform.install_steps.setup_dbs import run as run_setup_dbs
import shutil
import glob
from swimlane_platform.install_steps import PreFlightCheck


class SwimlaneInstaller(BaseWithLog):

    def __init__(self, config):
        # type: (Configuration) -> None
        super(SwimlaneInstaller, self).__init__(config)
        script_path = path.dirname(path.realpath(__file__))
        self.template_dir = find_remote_sibling(script_path, names.TEMPLATE_DIR)
        self.docker_manager = DockerManager(self.logger)

    @info_function_start_finish('Install Swimlane.')
    def run(self):
        # type: () -> None
        """
        Main Swimlane install method.
        """
        if not PreFlightCheck(self.config).run():
            return
        self.install_swimlane_files()
        self.get_swimlane_images()
        # order is important
        enable_web_ssl.run(self.config)
        enable_mongo_ssl.run(self.config)
        # ******************
        run_setup_environment(self.config)
        run_setup_dbs(self.config)
        run_set_permissions(self.config)
        run_add_file_encryption(self.config)
        enable_turbine.run(self.config)
        self.run_after_install()

    @info_function_start_finish('Installing Swimlane files.')
    def install_swimlane_files(self):
        """
        Make install directories and copy files there
        """
        self.create_if_does_not_exists(names.INSTALL_DIR)
        secrets_dir = path.join(names.INSTALL_DIR, names.SECRETS_SUB_FOLDER)
        self.create_if_does_not_exists(secrets_dir)
        db_init_dir = path.join(names.INSTALL_DIR, names.DB_INIT_SUB_FOLDER)
        self.create_if_does_not_exists(db_init_dir)
        self.copy_if_does_not_exists(self.template_dir, names.INSTALL_DIR, '*.yml')
        self.copy_if_does_not_exists(path.join(self.template_dir, names.DB_INIT_SUB_FOLDER), db_init_dir, '*.sh')
        self.copy_if_does_not_exists(path.join(self.template_dir, names.SECRETS_SUB_FOLDER), secrets_dir, '.*')

    @info_function_start_finish('Getting Swimlane docker images.')
    def get_swimlane_images(self):
        """Loads or pulls docker images required by Swimlane.
        In case of offline, analyses the images bundled, until build will know or care about repository
        re-tags nexus images. In case of online, analyses docker-compose file template and decides
        which version to pull.
        In both cases modifies docker-compose template to use correct images.
        """
        docker_compose_file = path.join(names.INSTALL_DIR, names.DOCKER_COMPOSE_FILE)
        docker_compose_file_manager = DockerComposeFileManager(self.logger, docker_compose_file)
        repository = names.DEV_REPOSITORY if self.config.args.dev else ''
        version = str(self.config.args.version) if self.config.args.version else self.latest_version()
        if self.config.args.installer_is_offline:
            images_file = path.join(self.config.args.extracted_files_folder, names.DOCKER_IMAGE_ARCHIVE_NAME)
            images = self.docker_manager.image_load(images_file)
            if not self.config.args.dev:
                for tag in (str(image.tags[0]) for image in images):
                    if tag.startswith(names.DEV_REPOSITORY):
                        new_tag = tag.replace(names.DEV_REPOSITORY, repository)
                        self.docker_manager.image_re_tag(tag, new_tag)
                        version = new_tag.split(':')[-1]
            for tag in (str(image.tags[0]) for image in images):
                self.logger.verbose("{image} loaded from archive.".format(image=tag))
        else:
            template_image_tags = docker_compose_file_manager.get_image_tags()
            for template_image_tag in template_image_tags:
                image_tag = template_image_tag \
                    .replace(names.TEMPLATE_REPOSITORY_TAG, repository) \
                    .replace(names.TEMPLATE_VERSION_TAG, version)
                self.docker_manager.image_pull(image_tag)
                self.logger.verbose("{image} docker image pulled.".format(image=image_tag))

        docker_compose_file_manager.replace_in_file_and_reload(names.TEMPLATE_REPOSITORY_TAG, repository)
        docker_compose_file_manager.replace_in_file_and_reload(names.TEMPLATE_VERSION_TAG, version)

    @staticmethod
    def latest_version():
        # type: () -> str
        """
        Returns the latest version of swimlane platform.
        :return: swimlane platform package version.
        """
        version_string = pkg_resources.get_distribution(swimlane_platform.__name__).version
        version = pypi_version_to_semver(version_string)
        if version:
            return str(version)
        else:
            raise ValidationException('Cannot parse package version {version} to Swimlane'.format(version=version))

    @info_function_start_finish('After installation.')
    def run_after_install(self):
        # type: () -> None
        """
        Prompts the user and starts or leaves containers as is.
        """
        questions = [{
            'type': 'confirm',
            'name': 'start_swimlane',
            'message': 'Would you like to start the Swimlane services?'
        }]
        self.config.collect(questions)
        if self.config.args.start_swimlane:
            docker_compose_file = path.join(names.INSTALL_DIR, names.DOCKER_COMPOSE_FILE)
            DockerComposeManager(self.logger, docker_compose_file).docker_compose_up()
        else:
            self.logger.info(
                'To start the Swimlane Services manually, cd /opt/swimlane and run "docker-compose up -d".')

    @debug_function_args
    def create_if_does_not_exists(self, dir_path):
        """Creates directory first checking for it's existence."""
        if not path.exists(dir_path):
            os.mkdir(dir_path)
            self.logger.verbose("{source} directory created.".format(source=dir_path))

    @debug_function_args
    def copy_if_does_not_exists(self, source, target, mask):
        for filepath in glob.glob1(source, mask):
            if not path.exists(path.join(target, filepath)):
                source_file = path.join(source, filepath)
                shutil.copy2(source_file, target)
                self.logger.verbose("File {source} copied to {target}".format(source=source_file, target=target))


def run(config):
    # type: (Configuration) -> None
    # noinspection PyCompatibility
    questions = [
        {
            'type': 'confirm',
            'name': 'installer_is_offline',
            'message': 'Are you installing offline?'
        },
        {
            'type': 'input',
            'name': 'extracted_files_folder',
            'message': 'Where are the extracted files located?',
            'when': lambda a: 'installer_is_offline' in a and a['installer_is_offline'],
            'default': u'.',
            'validate': PathExistsValidator
        },
        {
            'type': 'input',
            'name': 'version',
            'message': 'For development installation which version do you want to use?',
            'default': unicode(SwimlaneInstaller.latest_version()),
            'validate': VersionValidator,
            'when': lambda a: a.get('dev') and not a.get('installer_is_offline')
        }
    ]
    questions.extend(web_ssl_questions)
    config.collect(questions)
    if config.args.web_ssl_self_signed:
        config.collect(ssl_questions)
    SwimlaneInstaller(config).run()
